# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.fleetify.routes import (
    StepCreateResponse,
    StepDeleteResponse,
    StepUpdateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSteps:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        )
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
            address='"address": "503, Dublin Drive, Los Angeles, California - 500674",',
            completion_mode="`manual`",
            document_template_id="document_template_id",
            duration=0,
            geofence_config={
                "radius": 0,
                "type": "circle",
            },
            meta={
                "customer_name": '"customer_name": "Chandler Bing"',
                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
            },
        )
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.steps.with_raw_response.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = response.parse()
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.steps.with_streaming_response.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = response.parse()
            assert_matches_type(StepCreateResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_create(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.create(
                route_id="",
                key="key",
                arrival=0,
                location=[0],
                position=0,
                type="`start`",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        )
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
            address='"address": "503, Dublin Drive, Los Angeles, California - 500674",',
            completion_mode="`manual`",
            document_template_id="document_template_id",
            duration=0,
            geofence_config={
                "radius": 0,
                "type": "circle",
            },
            location=[0],
            meta={
                "customer_name": '"customer_name": "Chandler Bing"',
                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
            },
            type="`start`",
        )
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.steps.with_raw_response.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = response.parse()
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.steps.with_streaming_response.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = response.parse()
            assert_matches_type(StepUpdateResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.update(
                step_id="stepID",
                route_id="",
                key="key",
                arrival=0,
                position=0,
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.update(
                step_id="",
                route_id="routeID",
                key="key",
                arrival=0,
                position=0,
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )
        assert_matches_type(StepDeleteResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.steps.with_raw_response.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = response.parse()
        assert_matches_type(StepDeleteResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.steps.with_streaming_response.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = response.parse()
            assert_matches_type(StepDeleteResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.delete(
                step_id="stepID",
                route_id="",
                key="key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.delete(
                step_id="",
                route_id="routeID",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_complete(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )
        assert step is None

    @pytest.mark.skip()
    @parametrize
    def test_method_complete_with_all_params(self, client: NextbillionSDK) -> None:
        step = client.fleetify.routes.steps.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
            document={},
            mode="mode",
            status="status",
        )
        assert step is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_complete(self, client: NextbillionSDK) -> None:
        response = client.fleetify.routes.steps.with_raw_response.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = response.parse()
        assert step is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_complete(self, client: NextbillionSDK) -> None:
        with client.fleetify.routes.steps.with_streaming_response.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = response.parse()
            assert step is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_complete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.complete(
                step_id="stepID",
                route_id="",
                key="key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            client.fleetify.routes.steps.with_raw_response.complete(
                step_id="",
                route_id="routeID",
                key="key",
            )


class TestAsyncSteps:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        )
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
            address='"address": "503, Dublin Drive, Los Angeles, California - 500674",',
            completion_mode="`manual`",
            document_template_id="document_template_id",
            duration=0,
            geofence_config={
                "radius": 0,
                "type": "circle",
            },
            meta={
                "customer_name": '"customer_name": "Chandler Bing"',
                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
            },
        )
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.steps.with_raw_response.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = await response.parse()
        assert_matches_type(StepCreateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.steps.with_streaming_response.create(
            route_id="routeID",
            key="key",
            arrival=0,
            location=[0],
            position=0,
            type="`start`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = await response.parse()
            assert_matches_type(StepCreateResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_create(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.create(
                route_id="",
                key="key",
                arrival=0,
                location=[0],
                position=0,
                type="`start`",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        )
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
            address='"address": "503, Dublin Drive, Los Angeles, California - 500674",',
            completion_mode="`manual`",
            document_template_id="document_template_id",
            duration=0,
            geofence_config={
                "radius": 0,
                "type": "circle",
            },
            location=[0],
            meta={
                "customer_name": '"customer_name": "Chandler Bing"',
                "customer_phone_number": '"customer_phone_number": "+1 707 234 1234"',
                "instructions": '"instructions": "Customer asked not to ring the doorbell."',
            },
            type="`start`",
        )
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.steps.with_raw_response.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = await response.parse()
        assert_matches_type(StepUpdateResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.steps.with_streaming_response.update(
            step_id="stepID",
            route_id="routeID",
            key="key",
            arrival=0,
            position=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = await response.parse()
            assert_matches_type(StepUpdateResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.update(
                step_id="stepID",
                route_id="",
                key="key",
                arrival=0,
                position=0,
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.update(
                step_id="",
                route_id="routeID",
                key="key",
                arrival=0,
                position=0,
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )
        assert_matches_type(StepDeleteResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.steps.with_raw_response.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = await response.parse()
        assert_matches_type(StepDeleteResponse, step, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.steps.with_streaming_response.delete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = await response.parse()
            assert_matches_type(StepDeleteResponse, step, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.delete(
                step_id="stepID",
                route_id="",
                key="key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.delete(
                step_id="",
                route_id="routeID",
                key="key",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_complete(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )
        assert step is None

    @pytest.mark.skip()
    @parametrize
    async def test_method_complete_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        step = await async_client.fleetify.routes.steps.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
            document={},
            mode="mode",
            status="status",
        )
        assert step is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_complete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.fleetify.routes.steps.with_raw_response.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        step = await response.parse()
        assert step is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_complete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.fleetify.routes.steps.with_streaming_response.complete(
            step_id="stepID",
            route_id="routeID",
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            step = await response.parse()
            assert step is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_complete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `route_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.complete(
                step_id="stepID",
                route_id="",
                key="key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            await async_client.fleetify.routes.steps.with_raw_response.complete(
                step_id="",
                route_id="routeID",
                key="key",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..types import (
    restriction_list_params,
    restriction_create_params,
    restriction_delete_params,
    restriction_update_params,
    restriction_retrieve_params,
    restriction_set_state_params,
    restriction_list_by_bbox_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.rich_group_response import RichGroupResponse
from ..types.restriction_list_response import RestrictionListResponse
from ..types.restriction_delete_response import RestrictionDeleteResponse
from ..types.restriction_list_by_bbox_response import RestrictionListByBboxResponse

__all__ = ["RestrictionsResource", "AsyncRestrictionsResource"]


class RestrictionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RestrictionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return RestrictionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RestrictionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return RestrictionsResourceWithStreamingResponse(self)

    def create(
        self,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"],
        *,
        key: str,
        area: str,
        name: str,
        latlon: bool | NotGiven = NOT_GIVEN,
        comment: str | NotGiven = NOT_GIVEN,
        direction: Literal["`forward`", "`backward`", "`both`"] | NotGiven = NOT_GIVEN,
        end_time: float | NotGiven = NOT_GIVEN,
        geofence: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        height: int | NotGiven = NOT_GIVEN,
        length: int | NotGiven = NOT_GIVEN,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        repeat_on: str | NotGiven = NOT_GIVEN,
        segments: Iterable[restriction_create_params.Segment] | NotGiven = NOT_GIVEN,
        speed: float | NotGiven = NOT_GIVEN,
        speed_limit: float | NotGiven = NOT_GIVEN,
        start_time: float | NotGiven = NOT_GIVEN,
        tracks: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        turns: Iterable[restriction_create_params.Turn] | NotGiven = NOT_GIVEN,
        weight: int | NotGiven = NOT_GIVEN,
        width: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Create a new restriction

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          area: Specify the area name. It represents a region where restrictions can be applied.
              This is a custom field and it is recommended for the users to check with
              [NextBillion.ai](www.nextbillion.ai) support for the right value. Alternatively,
              users can invoke the _[Areas](#supported-areas)_ method to get a list of
              available areas for them.

          name: Specify a custom, descriptive name for the restriction.

          latlon: Use this parameter to decide the format for specifying the geofence coordinates.
              If true, then the coordinates of geofence can be specified as
              "latitude,longitude" format, otherwise they should be specified in
              "longitude,latitude" format.

          comment: Use this parameter to add any custom information about the restriction being
              created.

          direction: Represents the traffic direction on the segments to which the restriction will
              be applied.

          end_time: Provide a UNIX epoch timestamp in seconds, representing the time when the
              restriction should cease to be in-effect.

          geofence: An array of coordinates denoting the boundary of an area in which the
              restrictions are to be applied. The format in which coordinates should be listed
              is defined by the `latlon` field.

              Geofences can be used to create all restriction types, except for a `turn` type
              restriction. Please note that `segments` is not required when using `geofence`
              to create restrictions.

          height: Specify the maximum truck height, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          length: Specify the maximum truck length, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          mode: Provide the driving modes for which the restriction should be effective. If the
              value is an empty array or if it is not provided then the restriction would be
              applied for all modes.

          repeat_on: It represents the days and times when the restriction is in effect. Users can
              use this property to set recurring or one-time restrictions as per the
              [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours) for
              specifying the recurring schedule of the restriction.

              Please provided values as per the local time of the region where the restriction
              is being applied.

          segments: An array of objects to collect the details of the segments of a road on which
              the restriction has to be applied. Each object corresponds to a new segment.

              Please note that `segments` is mandatory for all `restrtiction_type` except
              `turn`.

          speed: Provide the the fixed speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `fixedspeed`.

          speed_limit: Provide the the maximum speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `maxspeed`.

          start_time: Provide a UNIX epoch timestamp in seconds, representing the start time for the
              restriction to be in-effect.

          tracks: Specify a sequence of coordinates (track) where the restriction is to be
              applied. The coordinates will be snapped to nearest road. Please note when using
              `tracks`, `segments` and `turns` are not required.

          turns: An array of objects to collect the details of the turns of a road on which the
              restriction has to be applied. Each object corresponds to a new turn.

              Please note that `turns` is mandatory for when `restrtiction_type=turn`.

          weight: Specify the maximum truck weight, in kilograms, that the restriction will allow.
              A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided for is `truck` restriction type.

          width: Specify the maximum truck width, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not restriction_type:
            raise ValueError(f"Expected a non-empty value for `restriction_type` but received {restriction_type!r}")
        return self._post(
            f"/restrictions/{restriction_type}",
            body=maybe_transform(
                {
                    "area": area,
                    "name": name,
                    "comment": comment,
                    "direction": direction,
                    "end_time": end_time,
                    "geofence": geofence,
                    "height": height,
                    "length": length,
                    "mode": mode,
                    "repeat_on": repeat_on,
                    "segments": segments,
                    "speed": speed,
                    "speed_limit": speed_limit,
                    "start_time": start_time,
                    "tracks": tracks,
                    "turns": turns,
                    "weight": weight,
                    "width": width,
                },
                restriction_create_params.RestrictionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "latlon": latlon,
                    },
                    restriction_create_params.RestrictionCreateParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    def retrieve(
        self,
        id: int,
        *,
        key: str,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Get a restriction by id

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          transform: a internal parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            f"/restrictions/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "transform": transform,
                    },
                    restriction_retrieve_params.RestrictionRetrieveParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    def update(
        self,
        id: int,
        *,
        key: str,
        area: str,
        name: str,
        latlon: bool | NotGiven = NOT_GIVEN,
        comment: str | NotGiven = NOT_GIVEN,
        direction: Literal["`forward`", "`backward`", "`both`"] | NotGiven = NOT_GIVEN,
        end_time: float | NotGiven = NOT_GIVEN,
        geofence: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        height: int | NotGiven = NOT_GIVEN,
        length: int | NotGiven = NOT_GIVEN,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        repeat_on: str | NotGiven = NOT_GIVEN,
        segments: Iterable[restriction_update_params.Segment] | NotGiven = NOT_GIVEN,
        speed: float | NotGiven = NOT_GIVEN,
        speed_limit: float | NotGiven = NOT_GIVEN,
        start_time: float | NotGiven = NOT_GIVEN,
        tracks: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        turns: Iterable[restriction_update_params.Turn] | NotGiven = NOT_GIVEN,
        weight: int | NotGiven = NOT_GIVEN,
        width: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Update a restriction

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          area: Specify the area name. It represents a region where restrictions can be applied.
              This is a custom field and it is recommended for the users to check with
              [NextBillion.ai](www.nextbillion.ai) support for the right value. Alternatively,
              users can invoke the _[Areas](#supported-areas)_ method to get a list of
              available areas for them.

          name: Specify a custom, descriptive name for the restriction.

          latlon: Use this parameter to decide the format for specifying the geofence coordinates.
              If true, then the coordinates of geofence can be specified as
              "latitude,longitude" format, otherwise they should be specified in
              "longitude,latitude" format.

          comment: Use this parameter to add any custom information about the restriction being
              created.

          direction: Represents the traffic direction on the segments to which the restriction will
              be applied.

          end_time: Provide a UNIX epoch timestamp in seconds, representing the time when the
              restriction should cease to be in-effect.

          geofence: An array of coordinates denoting the boundary of an area in which the
              restrictions are to be applied. The format in which coordinates should be listed
              is defined by the `latlon` field.

              Geofences can be used to create all restriction types, except for a `turn` type
              restriction. Please note that `segments` is not required when using `geofence`
              to create restrictions.

          height: Specify the maximum truck height, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          length: Specify the maximum truck length, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          mode: Provide the driving modes for which the restriction should be effective. If the
              value is an empty array or if it is not provided then the restriction would be
              applied for all modes.

          repeat_on: It represents the days and times when the restriction is in effect. Users can
              use this property to set recurring or one-time restrictions as per the
              [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours) for
              specifying the recurring schedule of the restriction.

              Please provided values as per the local time of the region where the restriction
              is being applied.

          segments: An array of objects to collect the details of the segments of a road on which
              the restriction has to be applied. Each object corresponds to a new segment.

              Please note that `segments` is mandatory for all `restrtiction_type` except
              `turn`.

          speed: Provide the the fixed speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `fixedspeed`.

          speed_limit: Provide the the maximum speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `maxspeed`.

          start_time: Provide a UNIX epoch timestamp in seconds, representing the start time for the
              restriction to be in-effect.

          tracks: Specify a sequence of coordinates (track) where the restriction is to be
              applied. The coordinates will be snapped to nearest road. Please note when using
              `tracks`, `segments` and `turns` are not required.

          turns: An array of objects to collect the details of the turns of a road on which the
              restriction has to be applied. Each object corresponds to a new turn.

              Please note that `turns` is mandatory for when `restrtiction_type=turn`.

          weight: Specify the maximum truck weight, in kilograms, that the restriction will allow.
              A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided for is `truck` restriction type.

          width: Specify the maximum truck width, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            f"/restrictions/{id}",
            body=maybe_transform(
                {
                    "area": area,
                    "name": name,
                    "comment": comment,
                    "direction": direction,
                    "end_time": end_time,
                    "geofence": geofence,
                    "height": height,
                    "length": length,
                    "mode": mode,
                    "repeat_on": repeat_on,
                    "segments": segments,
                    "speed": speed,
                    "speed_limit": speed_limit,
                    "start_time": start_time,
                    "tracks": tracks,
                    "turns": turns,
                    "weight": weight,
                    "width": width,
                },
                restriction_update_params.RestrictionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "latlon": latlon,
                    },
                    restriction_update_params.RestrictionUpdateParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    def list(
        self,
        *,
        area: str,
        key: str,
        limit: int,
        offset: int,
        mode: Literal["`0w`", "`2w`", "`3w`", "`4w`", "`6w`"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        restriction_type: Literal["`turn`", "`parking`", "`fixedspeed`", "`maxspeed`", "`closure`", "`truck`"]
        | NotGiven = NOT_GIVEN,
        source: Literal["rrt", "pbf"] | NotGiven = NOT_GIVEN,
        state: Literal["`enabled`", "`disabled`", "`deleted`"] | NotGiven = NOT_GIVEN,
        status: Literal["`active`", "`inactive`"] | NotGiven = NOT_GIVEN,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionListResponse:
        """Get the paginated list of restrictions

        Args:
          area: Specify the area name.

        It represents a region where restrictions can be applied.

              _The area it belongs to. See Area API_

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          limit: The number of restrictions to be returned in the response. Please note that if
              the `limit` is set to a number more than the total number of available
              restrictions, then all restrictions would be returned together.

          offset: An integer value indicating the number of items in the collection that need to
              be skipped in the response. Please note that the offset starts from 0, so the
              first item returned in the result would be the item at (offset + 1) position in
              collection.

              Users can use `offset` along with `limit` to implement paginated result.

          mode: Specify the modes of travel that the restriction pertains to.

          name: The name of the restriction. This should be same as that provided while creating
              or updating the restriction.

          restriction_type: Specify the type of restrictions to fetch.

          source: It represents where it comes from, currently the possible values include "rrt",
              "xsm"

          state: This parameter is used to filter restrictions based on their state i.e. whether
              the restriction is currently enabled, disabled, or deleted. For example, users
              can retrieve a list of all the deleted restrictions by setting `state=deleted`.

          status: Restrictions can be active or inactive at a given time by virtue of their
              nature. For example, maximum speed limits can be active on the roads leading to
              schools during school hours and be inactive afterwards or certain road closure
              restrictions be active during holidays/concerts and be inactive otherwise.

              Use this parameter to filter the restrictions based on their status at the time
              of making the request i.e. whether they are in force or not.

          transform: a internal parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/restrictions/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "area": area,
                        "key": key,
                        "limit": limit,
                        "offset": offset,
                        "mode": mode,
                        "name": name,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                        "transform": transform,
                    },
                    restriction_list_params.RestrictionListParams,
                ),
            ),
            cast_to=RestrictionListResponse,
        )

    def delete(
        self,
        id: int,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionDeleteResponse:
        """
        Delete a restriction by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            f"/restrictions/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, restriction_delete_params.RestrictionDeleteParams),
            ),
            cast_to=RestrictionDeleteResponse,
        )

    def list_by_bbox(
        self,
        *,
        key: str,
        max_lat: float,
        max_lon: float,
        min_lat: float,
        min_lon: float,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]
        | NotGiven = NOT_GIVEN,
        source: Literal["rrt", "pbf"] | NotGiven = NOT_GIVEN,
        state: Literal["`enabled`", "`disabled`", "`deleted`"] | NotGiven = NOT_GIVEN,
        status: Literal["`active`", "`inactive`"] | NotGiven = NOT_GIVEN,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionListByBboxResponse:
        """
        Get restrictions by bbox

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          max_lat: Specifies the maximum latitude value for the bounding box.

          max_lon: Specifies the maximum longitude value for the bounding box.

          min_lat: Specifies the minimum latitude value for the bounding box.

          min_lon: Specifies the minimum longitude value for the bounding box.

          mode: Specify the modes of travel that the restriction pertains to.

          restriction_type: Specify the type of restrictions to fetch.

          source: This parameter represents where the restriction comes from and cannot be
              modified by clients sending requests to the API endpoint.

              For example, an API endpoint that returns a list of restrictions could include
              the source parameter to indicate where each item comes from. This parameter can
              be useful for filtering, sorting, or grouping the results based on their source.

          state: This parameter is used to filter restrictions based on their state i.e. whether
              the restriction is currently enabled, disabled, or deleted. For example, users
              can retrieve a list of all the deleted restrictions by setting `state=deleted`.

          status: Restrictions can be active or inactive at a given time by virtue of their
              nature. For example, maximum speed limits can be active on the roads leading to
              schools during school hours and be inactive afterwards or certain road closure
              restrictions be active during holidays/concerts and be inactive otherwise.

              Use this parameter to filter the restrictions based on their status at the time
              of making the request i.e. whether they are in force or not.

          transform: This is internal parameter with a default value as `false`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/restrictions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "max_lat": max_lat,
                        "max_lon": max_lon,
                        "min_lat": min_lat,
                        "min_lon": min_lon,
                        "mode": mode,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                        "transform": transform,
                    },
                    restriction_list_by_bbox_params.RestrictionListByBboxParams,
                ),
            ),
            cast_to=RestrictionListByBboxResponse,
        )

    def set_state(
        self,
        id: int,
        *,
        key: str,
        state: Literal["`enabled`", "`disabled`", "`deleted`"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Set the state of a restriction by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          state: Use this field to specify the new `state` of the restriction. Please note that
              this method cannot update the state of restrictions that are currently in
              'deleted' state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            f"/restrictions/{id}/state",
            body=maybe_transform({"state": state}, restriction_set_state_params.RestrictionSetStateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, restriction_set_state_params.RestrictionSetStateParams),
            ),
            cast_to=RichGroupResponse,
        )


class AsyncRestrictionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRestrictionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRestrictionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRestrictionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncRestrictionsResourceWithStreamingResponse(self)

    async def create(
        self,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"],
        *,
        key: str,
        area: str,
        name: str,
        latlon: bool | NotGiven = NOT_GIVEN,
        comment: str | NotGiven = NOT_GIVEN,
        direction: Literal["`forward`", "`backward`", "`both`"] | NotGiven = NOT_GIVEN,
        end_time: float | NotGiven = NOT_GIVEN,
        geofence: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        height: int | NotGiven = NOT_GIVEN,
        length: int | NotGiven = NOT_GIVEN,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        repeat_on: str | NotGiven = NOT_GIVEN,
        segments: Iterable[restriction_create_params.Segment] | NotGiven = NOT_GIVEN,
        speed: float | NotGiven = NOT_GIVEN,
        speed_limit: float | NotGiven = NOT_GIVEN,
        start_time: float | NotGiven = NOT_GIVEN,
        tracks: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        turns: Iterable[restriction_create_params.Turn] | NotGiven = NOT_GIVEN,
        weight: int | NotGiven = NOT_GIVEN,
        width: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Create a new restriction

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          area: Specify the area name. It represents a region where restrictions can be applied.
              This is a custom field and it is recommended for the users to check with
              [NextBillion.ai](www.nextbillion.ai) support for the right value. Alternatively,
              users can invoke the _[Areas](#supported-areas)_ method to get a list of
              available areas for them.

          name: Specify a custom, descriptive name for the restriction.

          latlon: Use this parameter to decide the format for specifying the geofence coordinates.
              If true, then the coordinates of geofence can be specified as
              "latitude,longitude" format, otherwise they should be specified in
              "longitude,latitude" format.

          comment: Use this parameter to add any custom information about the restriction being
              created.

          direction: Represents the traffic direction on the segments to which the restriction will
              be applied.

          end_time: Provide a UNIX epoch timestamp in seconds, representing the time when the
              restriction should cease to be in-effect.

          geofence: An array of coordinates denoting the boundary of an area in which the
              restrictions are to be applied. The format in which coordinates should be listed
              is defined by the `latlon` field.

              Geofences can be used to create all restriction types, except for a `turn` type
              restriction. Please note that `segments` is not required when using `geofence`
              to create restrictions.

          height: Specify the maximum truck height, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          length: Specify the maximum truck length, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          mode: Provide the driving modes for which the restriction should be effective. If the
              value is an empty array or if it is not provided then the restriction would be
              applied for all modes.

          repeat_on: It represents the days and times when the restriction is in effect. Users can
              use this property to set recurring or one-time restrictions as per the
              [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours) for
              specifying the recurring schedule of the restriction.

              Please provided values as per the local time of the region where the restriction
              is being applied.

          segments: An array of objects to collect the details of the segments of a road on which
              the restriction has to be applied. Each object corresponds to a new segment.

              Please note that `segments` is mandatory for all `restrtiction_type` except
              `turn`.

          speed: Provide the the fixed speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `fixedspeed`.

          speed_limit: Provide the the maximum speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `maxspeed`.

          start_time: Provide a UNIX epoch timestamp in seconds, representing the start time for the
              restriction to be in-effect.

          tracks: Specify a sequence of coordinates (track) where the restriction is to be
              applied. The coordinates will be snapped to nearest road. Please note when using
              `tracks`, `segments` and `turns` are not required.

          turns: An array of objects to collect the details of the turns of a road on which the
              restriction has to be applied. Each object corresponds to a new turn.

              Please note that `turns` is mandatory for when `restrtiction_type=turn`.

          weight: Specify the maximum truck weight, in kilograms, that the restriction will allow.
              A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided for is `truck` restriction type.

          width: Specify the maximum truck width, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not restriction_type:
            raise ValueError(f"Expected a non-empty value for `restriction_type` but received {restriction_type!r}")
        return await self._post(
            f"/restrictions/{restriction_type}",
            body=await async_maybe_transform(
                {
                    "area": area,
                    "name": name,
                    "comment": comment,
                    "direction": direction,
                    "end_time": end_time,
                    "geofence": geofence,
                    "height": height,
                    "length": length,
                    "mode": mode,
                    "repeat_on": repeat_on,
                    "segments": segments,
                    "speed": speed,
                    "speed_limit": speed_limit,
                    "start_time": start_time,
                    "tracks": tracks,
                    "turns": turns,
                    "weight": weight,
                    "width": width,
                },
                restriction_create_params.RestrictionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "latlon": latlon,
                    },
                    restriction_create_params.RestrictionCreateParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    async def retrieve(
        self,
        id: int,
        *,
        key: str,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Get a restriction by id

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          transform: a internal parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            f"/restrictions/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "transform": transform,
                    },
                    restriction_retrieve_params.RestrictionRetrieveParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    async def update(
        self,
        id: int,
        *,
        key: str,
        area: str,
        name: str,
        latlon: bool | NotGiven = NOT_GIVEN,
        comment: str | NotGiven = NOT_GIVEN,
        direction: Literal["`forward`", "`backward`", "`both`"] | NotGiven = NOT_GIVEN,
        end_time: float | NotGiven = NOT_GIVEN,
        geofence: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        height: int | NotGiven = NOT_GIVEN,
        length: int | NotGiven = NOT_GIVEN,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        repeat_on: str | NotGiven = NOT_GIVEN,
        segments: Iterable[restriction_update_params.Segment] | NotGiven = NOT_GIVEN,
        speed: float | NotGiven = NOT_GIVEN,
        speed_limit: float | NotGiven = NOT_GIVEN,
        start_time: float | NotGiven = NOT_GIVEN,
        tracks: Iterable[Iterable[float]] | NotGiven = NOT_GIVEN,
        turns: Iterable[restriction_update_params.Turn] | NotGiven = NOT_GIVEN,
        weight: int | NotGiven = NOT_GIVEN,
        width: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Update a restriction

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          area: Specify the area name. It represents a region where restrictions can be applied.
              This is a custom field and it is recommended for the users to check with
              [NextBillion.ai](www.nextbillion.ai) support for the right value. Alternatively,
              users can invoke the _[Areas](#supported-areas)_ method to get a list of
              available areas for them.

          name: Specify a custom, descriptive name for the restriction.

          latlon: Use this parameter to decide the format for specifying the geofence coordinates.
              If true, then the coordinates of geofence can be specified as
              "latitude,longitude" format, otherwise they should be specified in
              "longitude,latitude" format.

          comment: Use this parameter to add any custom information about the restriction being
              created.

          direction: Represents the traffic direction on the segments to which the restriction will
              be applied.

          end_time: Provide a UNIX epoch timestamp in seconds, representing the time when the
              restriction should cease to be in-effect.

          geofence: An array of coordinates denoting the boundary of an area in which the
              restrictions are to be applied. The format in which coordinates should be listed
              is defined by the `latlon` field.

              Geofences can be used to create all restriction types, except for a `turn` type
              restriction. Please note that `segments` is not required when using `geofence`
              to create restrictions.

          height: Specify the maximum truck height, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          length: Specify the maximum truck length, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          mode: Provide the driving modes for which the restriction should be effective. If the
              value is an empty array or if it is not provided then the restriction would be
              applied for all modes.

          repeat_on: It represents the days and times when the restriction is in effect. Users can
              use this property to set recurring or one-time restrictions as per the
              [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours) for
              specifying the recurring schedule of the restriction.

              Please provided values as per the local time of the region where the restriction
              is being applied.

          segments: An array of objects to collect the details of the segments of a road on which
              the restriction has to be applied. Each object corresponds to a new segment.

              Please note that `segments` is mandatory for all `restrtiction_type` except
              `turn`.

          speed: Provide the the fixed speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `fixedspeed`.

          speed_limit: Provide the the maximum speed of the segment where the restriction needs to be
              applied. Please note that this parameter is mandatory when the `restrictionType`
              is `maxspeed`.

          start_time: Provide a UNIX epoch timestamp in seconds, representing the start time for the
              restriction to be in-effect.

          tracks: Specify a sequence of coordinates (track) where the restriction is to be
              applied. The coordinates will be snapped to nearest road. Please note when using
              `tracks`, `segments` and `turns` are not required.

          turns: An array of objects to collect the details of the turns of a road on which the
              restriction has to be applied. Each object corresponds to a new turn.

              Please note that `turns` is mandatory for when `restrtiction_type=turn`.

          weight: Specify the maximum truck weight, in kilograms, that the restriction will allow.
              A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided for is `truck` restriction type.

          width: Specify the maximum truck width, in centimeter, that will be allowed under the
              restriction. A value of 0 indicates no limit.

              Please note this parameter is effective only when `restriction_type` is `truck`.
              At least one of truck parameters - `weight`, `height`, `width` and `truck` -
              needs to be provided when restriction type is `truck`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            f"/restrictions/{id}",
            body=await async_maybe_transform(
                {
                    "area": area,
                    "name": name,
                    "comment": comment,
                    "direction": direction,
                    "end_time": end_time,
                    "geofence": geofence,
                    "height": height,
                    "length": length,
                    "mode": mode,
                    "repeat_on": repeat_on,
                    "segments": segments,
                    "speed": speed,
                    "speed_limit": speed_limit,
                    "start_time": start_time,
                    "tracks": tracks,
                    "turns": turns,
                    "weight": weight,
                    "width": width,
                },
                restriction_update_params.RestrictionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "latlon": latlon,
                    },
                    restriction_update_params.RestrictionUpdateParams,
                ),
            ),
            cast_to=RichGroupResponse,
        )

    async def list(
        self,
        *,
        area: str,
        key: str,
        limit: int,
        offset: int,
        mode: Literal["`0w`", "`2w`", "`3w`", "`4w`", "`6w`"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        restriction_type: Literal["`turn`", "`parking`", "`fixedspeed`", "`maxspeed`", "`closure`", "`truck`"]
        | NotGiven = NOT_GIVEN,
        source: Literal["rrt", "pbf"] | NotGiven = NOT_GIVEN,
        state: Literal["`enabled`", "`disabled`", "`deleted`"] | NotGiven = NOT_GIVEN,
        status: Literal["`active`", "`inactive`"] | NotGiven = NOT_GIVEN,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionListResponse:
        """Get the paginated list of restrictions

        Args:
          area: Specify the area name.

        It represents a region where restrictions can be applied.

              _The area it belongs to. See Area API_

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          limit: The number of restrictions to be returned in the response. Please note that if
              the `limit` is set to a number more than the total number of available
              restrictions, then all restrictions would be returned together.

          offset: An integer value indicating the number of items in the collection that need to
              be skipped in the response. Please note that the offset starts from 0, so the
              first item returned in the result would be the item at (offset + 1) position in
              collection.

              Users can use `offset` along with `limit` to implement paginated result.

          mode: Specify the modes of travel that the restriction pertains to.

          name: The name of the restriction. This should be same as that provided while creating
              or updating the restriction.

          restriction_type: Specify the type of restrictions to fetch.

          source: It represents where it comes from, currently the possible values include "rrt",
              "xsm"

          state: This parameter is used to filter restrictions based on their state i.e. whether
              the restriction is currently enabled, disabled, or deleted. For example, users
              can retrieve a list of all the deleted restrictions by setting `state=deleted`.

          status: Restrictions can be active or inactive at a given time by virtue of their
              nature. For example, maximum speed limits can be active on the roads leading to
              schools during school hours and be inactive afterwards or certain road closure
              restrictions be active during holidays/concerts and be inactive otherwise.

              Use this parameter to filter the restrictions based on their status at the time
              of making the request i.e. whether they are in force or not.

          transform: a internal parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/restrictions/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "area": area,
                        "key": key,
                        "limit": limit,
                        "offset": offset,
                        "mode": mode,
                        "name": name,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                        "transform": transform,
                    },
                    restriction_list_params.RestrictionListParams,
                ),
            ),
            cast_to=RestrictionListResponse,
        )

    async def delete(
        self,
        id: int,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionDeleteResponse:
        """
        Delete a restriction by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            f"/restrictions/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, restriction_delete_params.RestrictionDeleteParams),
            ),
            cast_to=RestrictionDeleteResponse,
        )

    async def list_by_bbox(
        self,
        *,
        key: str,
        max_lat: float,
        max_lon: float,
        min_lat: float,
        min_lon: float,
        mode: List[Literal["0w", "2w", "3w", "4w", "6w"]] | NotGiven = NOT_GIVEN,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]
        | NotGiven = NOT_GIVEN,
        source: Literal["rrt", "pbf"] | NotGiven = NOT_GIVEN,
        state: Literal["`enabled`", "`disabled`", "`deleted`"] | NotGiven = NOT_GIVEN,
        status: Literal["`active`", "`inactive`"] | NotGiven = NOT_GIVEN,
        transform: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionListByBboxResponse:
        """
        Get restrictions by bbox

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          max_lat: Specifies the maximum latitude value for the bounding box.

          max_lon: Specifies the maximum longitude value for the bounding box.

          min_lat: Specifies the minimum latitude value for the bounding box.

          min_lon: Specifies the minimum longitude value for the bounding box.

          mode: Specify the modes of travel that the restriction pertains to.

          restriction_type: Specify the type of restrictions to fetch.

          source: This parameter represents where the restriction comes from and cannot be
              modified by clients sending requests to the API endpoint.

              For example, an API endpoint that returns a list of restrictions could include
              the source parameter to indicate where each item comes from. This parameter can
              be useful for filtering, sorting, or grouping the results based on their source.

          state: This parameter is used to filter restrictions based on their state i.e. whether
              the restriction is currently enabled, disabled, or deleted. For example, users
              can retrieve a list of all the deleted restrictions by setting `state=deleted`.

          status: Restrictions can be active or inactive at a given time by virtue of their
              nature. For example, maximum speed limits can be active on the roads leading to
              schools during school hours and be inactive afterwards or certain road closure
              restrictions be active during holidays/concerts and be inactive otherwise.

              Use this parameter to filter the restrictions based on their status at the time
              of making the request i.e. whether they are in force or not.

          transform: This is internal parameter with a default value as `false`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/restrictions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "max_lat": max_lat,
                        "max_lon": max_lon,
                        "min_lat": min_lat,
                        "min_lon": min_lon,
                        "mode": mode,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                        "transform": transform,
                    },
                    restriction_list_by_bbox_params.RestrictionListByBboxParams,
                ),
            ),
            cast_to=RestrictionListByBboxResponse,
        )

    async def set_state(
        self,
        id: int,
        *,
        key: str,
        state: Literal["`enabled`", "`disabled`", "`deleted`"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RichGroupResponse:
        """
        Set the state of a restriction by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          state: Use this field to specify the new `state` of the restriction. Please note that
              this method cannot update the state of restrictions that are currently in
              'deleted' state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            f"/restrictions/{id}/state",
            body=await async_maybe_transform({"state": state}, restriction_set_state_params.RestrictionSetStateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, restriction_set_state_params.RestrictionSetStateParams),
            ),
            cast_to=RichGroupResponse,
        )


class RestrictionsResourceWithRawResponse:
    def __init__(self, restrictions: RestrictionsResource) -> None:
        self._restrictions = restrictions

        self.create = to_raw_response_wrapper(
            restrictions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            restrictions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            restrictions.update,
        )
        self.list = to_raw_response_wrapper(
            restrictions.list,
        )
        self.delete = to_raw_response_wrapper(
            restrictions.delete,
        )
        self.list_by_bbox = to_raw_response_wrapper(
            restrictions.list_by_bbox,
        )
        self.set_state = to_raw_response_wrapper(
            restrictions.set_state,
        )


class AsyncRestrictionsResourceWithRawResponse:
    def __init__(self, restrictions: AsyncRestrictionsResource) -> None:
        self._restrictions = restrictions

        self.create = async_to_raw_response_wrapper(
            restrictions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            restrictions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            restrictions.update,
        )
        self.list = async_to_raw_response_wrapper(
            restrictions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            restrictions.delete,
        )
        self.list_by_bbox = async_to_raw_response_wrapper(
            restrictions.list_by_bbox,
        )
        self.set_state = async_to_raw_response_wrapper(
            restrictions.set_state,
        )


class RestrictionsResourceWithStreamingResponse:
    def __init__(self, restrictions: RestrictionsResource) -> None:
        self._restrictions = restrictions

        self.create = to_streamed_response_wrapper(
            restrictions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            restrictions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            restrictions.update,
        )
        self.list = to_streamed_response_wrapper(
            restrictions.list,
        )
        self.delete = to_streamed_response_wrapper(
            restrictions.delete,
        )
        self.list_by_bbox = to_streamed_response_wrapper(
            restrictions.list_by_bbox,
        )
        self.set_state = to_streamed_response_wrapper(
            restrictions.set_state,
        )


class AsyncRestrictionsResourceWithStreamingResponse:
    def __init__(self, restrictions: AsyncRestrictionsResource) -> None:
        self._restrictions = restrictions

        self.create = async_to_streamed_response_wrapper(
            restrictions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            restrictions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            restrictions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            restrictions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            restrictions.delete,
        )
        self.list_by_bbox = async_to_streamed_response_wrapper(
            restrictions.list_by_bbox,
        )
        self.set_state = async_to_streamed_response_wrapper(
            restrictions.set_state,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .rich_group_response import RichGroupResponse

__all__ = ["RestrictionListByBboxResponse"]

RestrictionListByBboxResponse: TypeAlias = List[RichGroupResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MonitorListParams"]


class MonitorListParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results.
    """

    sort: str
    """Provide a single field to sort the results by.

    Only `updated_at` or `created_at` fields can be selected for ordering the
    results.

    By default, the result is sorted by `created_at` field in the descending order.
    Allowed values for specifying the order are `asc` for ascending order and `desc`
    for descending order.
    """

    tags: str
    """`tags` can be used to filter the monitors.

    Only those monitors which have all the `tags` provided here, will be included in
    the search result. In case multiple `tags` need to be specified, use `,` to
    separate them.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .geofence import Geofence
from ..._models import BaseModel

__all__ = ["BatchListResponse", "Data"]


class Data(BaseModel):
    list: List[Geofence]
    """
    An array of objects containing the details of the geofences returned matching
    the IDs provided. Each object represents one geofence.
    """


class BatchListResponse(BaseModel):
    data: Data

    status: str
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AssetBindParams"]


class AssetBindParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    device_id: Required[str]
    """Device ID to be linked to the `asset` identified by `id`.

    Please note that the device needs to be linked to an `asset` before using it in
    the _Upload locations of an Asset_ method for sending GPS information about the
    `asset`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .routing_response import RoutingResponse
from .routes.route_steps_response import RouteStepsResponse

__all__ = ["RouteCreateResponse", "Data", "DataDriver"]


class DataDriver(BaseModel):
    id: Optional[str] = None
    """
    Returns the ID of the driver as specified in the
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """

    email: Optional[str] = None
    """
    Returns the email of the driver as specified in the
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """

    fullname: Optional[str] = None
    """
    Returns the full name of the driver as specified in
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the dispatched route."""

    created_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this route dispatch
    request was created.
    """

    distance: Optional[int] = None
    """
    Returns the total route distance, in meters, for informative display in the
    driver app. It is the same as the value provided for `distance` field in the
    input request.
    """

    document_snapshot: Optional[List[object]] = None
    """
    Returns the details of the document that was specified in the input for
    collecting the proof-of-completion for all steps in the dispatched routes. Each
    object represents a new field in the document.
    """

    driver: Optional[DataDriver] = None
    """An object returning the details of the driver to whom the route was dispatched."""

    ro_request_id: Optional[str] = None
    """Returns the route optimization request ID which was used to dispatch the route.

    An empty string is returned if the corresponding input was not provided.
    """

    routing: Optional[RoutingResponse] = None
    """
    An object returning the routing characteristics that are used to generate the
    route and turn-by-turn navigation steps for the dispatched route. The route and
    navigation steps are available when driver uses the in-app navigation.

    Please note the routing characteristics returned here are the same as those
    configured in the input request. The fields which were not specified in the
    input will be returned as blanks.
    """

    short_id: Optional[str] = None
    """
    Returns a shorter unique ID of the dispatched route for easier referencing and
    displaying purposes.
    """

    steps: Optional[List[RouteStepsResponse]] = None
    """
    An array of objects containing the details of all steps to be performed as part
    of the dispatched route. Each object represents a single step during the route.
    """

    total_steps: Optional[int] = None
    """Returns the total number of steps in the dispatched route."""

    updated_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this route dispatch
    request was updated.
    """

    vehicle_id: Optional[str] = None
    """Returns the ID of the vehicle to which the route was dispatched.

    The vehicle ID returned here is the same as the one used in the route
    optimization request for the given vehicle. An empty string is returned if the
    `ro_request_id` was not provided in the input.
    """


class RouteCreateResponse(BaseModel):
    data: Optional[Data] = None
    """An array of objects containing the details of each step in the dispatched route.

    Each object represents a single step.
    """

    status: Optional[int] = None
    """Returns the status code of the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from ..pagination import Pagination

__all__ = [
    "EventListResponse",
    "Data",
    "DataList",
    "DataListPrevLocation",
    "DataListPrevLocationLocation",
    "DataListTriggeredLocation",
    "DataListTriggeredLocationLocation",
]


class DataListPrevLocationLocation(BaseModel):
    lat: Optional[float] = None
    """Latitude of the `prev_location` tracked for the `asset`."""

    lon: Optional[float] = None
    """Longitude of the `prev_location` tracked for the `asset`."""


class DataListPrevLocation(BaseModel):
    bearing: Optional[float] = None
    """
    If available, this property returns the heading of the `asset` from true north
    in clockwise direction, at the `prev_location` tracked for the `asset`.
    """

    location: Optional[DataListPrevLocationLocation] = None
    """`prev_location` information of the `asset`."""

    meta_data: Optional[object] = None
    """Returns the custom data added during the location information upload."""

    speed: Optional[float] = None
    """
    If available, this property returns the speed of the `asset`, in meters per
    second, at the `prev_location` of the `asset`.
    """

    timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds representing the time at which the
    `asset` was at the `prev_location`.
    """


class DataListTriggeredLocationLocation(BaseModel):
    lat: Optional[float] = None
    """Latitude of the `triggered_location` of the event."""

    lon: Optional[float] = None
    """Longitude of the `triggered_location` of the event."""


class DataListTriggeredLocation(BaseModel):
    bearing: Optional[float] = None
    """
    If available, this property returns the heading of the `asset` from true north
    in clockwise direction, when the event was triggered.
    """

    location: Optional[DataListTriggeredLocationLocation] = None
    """An object with information about the location at which the event was triggered."""

    meta_data: Optional[object] = None
    """Returns the custom data added during the location information upload."""

    speed: Optional[float] = None
    """
    If available, this property returns the speed of the `asset`, in meters per
    second, when the event was triggered.
    """

    timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds representing the time at which the
    `asset` was at the `triggered_location`.
    """


class DataList(BaseModel):
    asset_id: Optional[str] = None
    """ID of the `asset`.

    This is the same ID that was generated/provided at the time of creating the
    `asset`.
    """

    event_type: Optional[Literal["`enter`", "`exit`", "`speeding`", "`idle`"]] = None
    """Nature of the event triggered by the `asset`. It can have following values:

    - `enter`: When the `asset` enters a specific geofence

    - `exit`: When the `asset` moves out of a specific geofence.

    - `speeding`: When the `asset` exceeds the certain speed limit.

    - `idle`: When the `asset` exhibits idle or no activity.
    """

    extra: Optional[object] = None
    """Additional information about the event.

    Currently, this object returns the speed limit that was used to generate the
    over-speeding events, for a `speeding` type event.

    It is worth highlighting that, when the `use_admin_speed_limit` is `true`, the
    speed limit value will be obtained from the underlying road information.
    Whereas, if the `use_admin_speed_limit` is `false`, the speed limit will be
    equal to the `customer_speed_limit` value provided by the user when creating or
    updating the `monitor`.
    """

    geofence_id: Optional[str] = None
    """ID of the `geofence` associated with the event."""

    monitor_id: Optional[str] = None
    """ID of the `monitor` associated with the event."""

    monitor_tags: Optional[List[str]] = None
    """Tags associated with the `monitor`."""

    prev_location: Optional[DataListPrevLocation] = None
    """
    An object with details of the `asset` at the last tracked location before the
    event was triggered.
    """

    timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds representing the time at which the event
    was added/created.
    """

    triggered_location: Optional[DataListTriggeredLocation] = None
    """
    An object with details of the `asset` at the location where the event was
    triggered.
    """

    triggered_timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds representing the time at which the event
    was triggered.
    """


class Data(BaseModel):
    list: Optional[List[DataList]] = None
    """An array of objects with each object on the list representing one event."""

    page: Optional[Pagination] = None
    """An object with pagination details of the search results.

    Use this object to implement pagination in your application.
    """


class EventListResponse(BaseModel):
    data: Optional[Data] = None
    """
    An object containing the information about the event history for the requested
    `asset`.
    """

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .location import Location as Location
from .job_param import JobParam as JobParam
from .vehicle_param import VehicleParam as VehicleParam
from .location_param import LocationParam as LocationParam
from .shipment_param import ShipmentParam as ShipmentParam
from .v2_submit_params import V2SubmitParams as V2SubmitParams
from .v2_retrieve_result_params import V2RetrieveResultParams as V2RetrieveResultParams
from .v2_retrieve_result_response import V2RetrieveResultResponse as V2RetrieveResultResponse
from .driver_assignment_assign_params import DriverAssignmentAssignParams as DriverAssignmentAssignParams
from .driver_assignment_assign_response import DriverAssignmentAssignResponse as DriverAssignmentAssignResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from .._models import BaseModel

__all__ = ["AreaListResponse", "AreaListResponseItem"]


class AreaListResponseItem(BaseModel):
    code: Optional[str] = None
    """Returns the code for the available area."""

    modes: Optional[List[str]] = None
    """Returns available traveling modes for the given area."""

    name: Optional[str] = None
    """Returns the name of the available area."""

    timezone: Optional[float] = None
    """Returns the offset from UTC time."""


AreaListResponse: TypeAlias = List[AreaListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .routes.route_steps_request_param import RouteStepsRequestParam
from .routes.route_step_completion_mode import RouteStepCompletionMode

__all__ = ["RouteRedispatchParams", "Operation", "OperationData"]


class RouteRedispatchParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    operations: Required[Iterable[Operation]]
    """A collection of objects with details of the steps to be modified.

    Each object corresponds to a single step.
    """

    distance: float
    """Specify the distance of the route."""


class OperationData(TypedDict, total=False):
    completion_mode: RouteStepCompletionMode
    """Specify the mode of completion to be used for the step.

    Currently, following values are allowed:

    - `manual`: Steps must be marked as completed manually through the Driver App.
    - `geofence`: Steps are marked as completed automatically based on the entry
      conditions and geofence specified.
    - `geofence_manual_fallback`: Steps will be marked as completed automatically
      based on geofence and entry condition configurations but there will also be a
      provision for manually updating the status in case, geofence detection fails.
    """

    document_template_id: str
    """
    Specify the ID of the document template to be used for collecting proof of
    completion for the step. It would be applied to step which not be bind to
    document template. Use the
    [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
    create, read and manage the document templates.

    Please note that the document template ID can not be assigned to following step
    types - `start`, `end`, `break`, `layover`.
    """

    step: RouteStepsRequestParam

    step_id: str
    """Specify the ID of the step to be updated or deleted.

    Either one of `id` or `short_id` of the step can be provided. This input will be
    ignored when `operation: create` .
    """


class Operation(TypedDict, total=False):
    data: Required[OperationData]

    operation: Required[Literal["`create`", "`update`", "`delete`"]]
    """Specify the type of operation to be performed for the step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .map import (
    MapResource,
    AsyncMapResource,
    MapResourceWithRawResponse,
    AsyncMapResourceWithRawResponse,
    MapResourceWithStreamingResponse,
    AsyncMapResourceWithStreamingResponse,
)
from .mdm import (
    MdmResource,
    AsyncMdmResource,
    MdmResourceWithRawResponse,
    AsyncMdmResourceWithRawResponse,
    MdmResourceWithStreamingResponse,
    AsyncMdmResourceWithStreamingResponse,
)
from .areas import (
    AreasResource,
    AsyncAreasResource,
    AreasResourceWithRawResponse,
    AsyncAreasResourceWithRawResponse,
    AreasResourceWithStreamingResponse,
    AsyncAreasResourceWithStreamingResponse,
)
from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from .browse import (
    BrowseResource,
    AsyncBrowseResource,
    BrowseResourceWithRawResponse,
    AsyncBrowseResourceWithRawResponse,
    BrowseResourceWithStreamingResponse,
    AsyncBrowseResourceWithStreamingResponse,
)
from .lookup import (
    LookupResource,
    AsyncLookupResource,
    LookupResourceWithRawResponse,
    AsyncLookupResourceWithRawResponse,
    LookupResourceWithStreamingResponse,
    AsyncLookupResourceWithStreamingResponse,
)
from .skynet import (
    SkynetResource,
    AsyncSkynetResource,
    SkynetResourceWithRawResponse,
    AsyncSkynetResourceWithRawResponse,
    SkynetResourceWithStreamingResponse,
    AsyncSkynetResourceWithStreamingResponse,
)
from .geocode import (
    GeocodeResource,
    AsyncGeocodeResource,
    GeocodeResourceWithRawResponse,
    AsyncGeocodeResourceWithRawResponse,
    GeocodeResourceWithStreamingResponse,
    AsyncGeocodeResourceWithStreamingResponse,
)
from .discover import (
    DiscoverResource,
    AsyncDiscoverResource,
    DiscoverResourceWithRawResponse,
    AsyncDiscoverResourceWithRawResponse,
    DiscoverResourceWithStreamingResponse,
    AsyncDiscoverResourceWithStreamingResponse,
)
from .fleetify import (
    FleetifyResource,
    AsyncFleetifyResource,
    FleetifyResourceWithRawResponse,
    AsyncFleetifyResourceWithRawResponse,
    FleetifyResourceWithStreamingResponse,
    AsyncFleetifyResourceWithStreamingResponse,
)
from .geofence import (
    GeofenceResource,
    AsyncGeofenceResource,
    GeofenceResourceWithRawResponse,
    AsyncGeofenceResourceWithRawResponse,
    GeofenceResourceWithStreamingResponse,
    AsyncGeofenceResourceWithStreamingResponse,
)
from .isochrone import (
    IsochroneResource,
    AsyncIsochroneResource,
    IsochroneResourceWithRawResponse,
    AsyncIsochroneResourceWithRawResponse,
    IsochroneResourceWithStreamingResponse,
    AsyncIsochroneResourceWithStreamingResponse,
)
from .directions import (
    DirectionsResource,
    AsyncDirectionsResource,
    DirectionsResourceWithRawResponse,
    AsyncDirectionsResourceWithRawResponse,
    DirectionsResourceWithStreamingResponse,
    AsyncDirectionsResourceWithStreamingResponse,
)
from .navigation import (
    NavigationResource,
    AsyncNavigationResource,
    NavigationResourceWithRawResponse,
    AsyncNavigationResourceWithRawResponse,
    NavigationResourceWithStreamingResponse,
    AsyncNavigationResourceWithStreamingResponse,
)
from .postalcode import (
    PostalcodeResource,
    AsyncPostalcodeResource,
    PostalcodeResourceWithRawResponse,
    AsyncPostalcodeResourceWithRawResponse,
    PostalcodeResourceWithStreamingResponse,
    AsyncPostalcodeResourceWithStreamingResponse,
)
from .revgeocode import (
    RevgeocodeResource,
    AsyncRevgeocodeResource,
    RevgeocodeResourceWithRawResponse,
    AsyncRevgeocodeResourceWithRawResponse,
    RevgeocodeResourceWithStreamingResponse,
    AsyncRevgeocodeResourceWithStreamingResponse,
)
from .autosuggest import (
    AutosuggestResource,
    AsyncAutosuggestResource,
    AutosuggestResourceWithRawResponse,
    AsyncAutosuggestResourceWithRawResponse,
    AutosuggestResourceWithStreamingResponse,
    AsyncAutosuggestResourceWithStreamingResponse,
)
from .autocomplete import (
    AutocompleteResource,
    AsyncAutocompleteResource,
    AutocompleteResourceWithRawResponse,
    AsyncAutocompleteResourceWithRawResponse,
    AutocompleteResourceWithStreamingResponse,
    AsyncAutocompleteResourceWithStreamingResponse,
)
from .multigeocode import (
    MultigeocodeResource,
    AsyncMultigeocodeResource,
    MultigeocodeResourceWithRawResponse,
    AsyncMultigeocodeResourceWithRawResponse,
    MultigeocodeResourceWithStreamingResponse,
    AsyncMultigeocodeResourceWithStreamingResponse,
)
from .optimization import (
    OptimizationResource,
    AsyncOptimizationResource,
    OptimizationResourceWithRawResponse,
    AsyncOptimizationResourceWithRawResponse,
    OptimizationResourceWithStreamingResponse,
    AsyncOptimizationResourceWithStreamingResponse,
)
from .restrictions import (
    RestrictionsResource,
    AsyncRestrictionsResource,
    RestrictionsResourceWithRawResponse,
    AsyncRestrictionsResourceWithRawResponse,
    RestrictionsResourceWithStreamingResponse,
    AsyncRestrictionsResourceWithStreamingResponse,
)
from .route_report import (
    RouteReportResource,
    AsyncRouteReportResource,
    RouteReportResourceWithRawResponse,
    AsyncRouteReportResourceWithRawResponse,
    RouteReportResourceWithStreamingResponse,
    AsyncRouteReportResourceWithStreamingResponse,
)
from .snap_to_roads import (
    SnapToRoadsResource,
    AsyncSnapToRoadsResource,
    SnapToRoadsResourceWithRawResponse,
    AsyncSnapToRoadsResourceWithRawResponse,
    SnapToRoadsResourceWithStreamingResponse,
    AsyncSnapToRoadsResourceWithStreamingResponse,
)
from .distancematrix import (
    DistancematrixResource,
    AsyncDistancematrixResource,
    DistancematrixResourceWithRawResponse,
    AsyncDistancematrixResourceWithRawResponse,
    DistancematrixResourceWithStreamingResponse,
    AsyncDistancematrixResourceWithStreamingResponse,
)
from .restrictions_items import (
    RestrictionsItemsResource,
    AsyncRestrictionsItemsResource,
    RestrictionsItemsResourceWithRawResponse,
    AsyncRestrictionsItemsResourceWithRawResponse,
    RestrictionsItemsResourceWithStreamingResponse,
    AsyncRestrictionsItemsResourceWithStreamingResponse,
)

__all__ = [
    "FleetifyResource",
    "AsyncFleetifyResource",
    "FleetifyResourceWithRawResponse",
    "AsyncFleetifyResourceWithRawResponse",
    "FleetifyResourceWithStreamingResponse",
    "AsyncFleetifyResourceWithStreamingResponse",
    "SkynetResource",
    "AsyncSkynetResource",
    "SkynetResourceWithRawResponse",
    "AsyncSkynetResourceWithRawResponse",
    "SkynetResourceWithStreamingResponse",
    "AsyncSkynetResourceWithStreamingResponse",
    "GeocodeResource",
    "AsyncGeocodeResource",
    "GeocodeResourceWithRawResponse",
    "AsyncGeocodeResourceWithRawResponse",
    "GeocodeResourceWithStreamingResponse",
    "AsyncGeocodeResourceWithStreamingResponse",
    "OptimizationResource",
    "AsyncOptimizationResource",
    "OptimizationResourceWithRawResponse",
    "AsyncOptimizationResourceWithRawResponse",
    "OptimizationResourceWithStreamingResponse",
    "AsyncOptimizationResourceWithStreamingResponse",
    "GeofenceResource",
    "AsyncGeofenceResource",
    "GeofenceResourceWithRawResponse",
    "AsyncGeofenceResourceWithRawResponse",
    "GeofenceResourceWithStreamingResponse",
    "AsyncGeofenceResourceWithStreamingResponse",
    "DiscoverResource",
    "AsyncDiscoverResource",
    "DiscoverResourceWithRawResponse",
    "AsyncDiscoverResourceWithRawResponse",
    "DiscoverResourceWithStreamingResponse",
    "AsyncDiscoverResourceWithStreamingResponse",
    "BrowseResource",
    "AsyncBrowseResource",
    "BrowseResourceWithRawResponse",
    "AsyncBrowseResourceWithRawResponse",
    "BrowseResourceWithStreamingResponse",
    "AsyncBrowseResourceWithStreamingResponse",
    "MdmResource",
    "AsyncMdmResource",
    "MdmResourceWithRawResponse",
    "AsyncMdmResourceWithRawResponse",
    "MdmResourceWithStreamingResponse",
    "AsyncMdmResourceWithStreamingResponse",
    "IsochroneResource",
    "AsyncIsochroneResource",
    "IsochroneResourceWithRawResponse",
    "AsyncIsochroneResourceWithRawResponse",
    "IsochroneResourceWithStreamingResponse",
    "AsyncIsochroneResourceWithStreamingResponse",
    "RestrictionsResource",
    "AsyncRestrictionsResource",
    "RestrictionsResourceWithRawResponse",
    "AsyncRestrictionsResourceWithRawResponse",
    "RestrictionsResourceWithStreamingResponse",
    "AsyncRestrictionsResourceWithStreamingResponse",
    "RestrictionsItemsResource",
    "AsyncRestrictionsItemsResource",
    "RestrictionsItemsResourceWithRawResponse",
    "AsyncRestrictionsItemsResourceWithRawResponse",
    "RestrictionsItemsResourceWithStreamingResponse",
    "AsyncRestrictionsItemsResourceWithStreamingResponse",
    "DistancematrixResource",
    "AsyncDistancematrixResource",
    "DistancematrixResourceWithRawResponse",
    "AsyncDistancematrixResourceWithRawResponse",
    "DistancematrixResourceWithStreamingResponse",
    "AsyncDistancematrixResourceWithStreamingResponse",
    "AutocompleteResource",
    "AsyncAutocompleteResource",
    "AutocompleteResourceWithRawResponse",
    "AsyncAutocompleteResourceWithRawResponse",
    "AutocompleteResourceWithStreamingResponse",
    "AsyncAutocompleteResourceWithStreamingResponse",
    "NavigationResource",
    "AsyncNavigationResource",
    "NavigationResourceWithRawResponse",
    "AsyncNavigationResourceWithRawResponse",
    "NavigationResourceWithStreamingResponse",
    "AsyncNavigationResourceWithStreamingResponse",
    "MapResource",
    "AsyncMapResource",
    "MapResourceWithRawResponse",
    "AsyncMapResourceWithRawResponse",
    "MapResourceWithStreamingResponse",
    "AsyncMapResourceWithStreamingResponse",
    "AutosuggestResource",
    "AsyncAutosuggestResource",
    "AutosuggestResourceWithRawResponse",
    "AsyncAutosuggestResourceWithRawResponse",
    "AutosuggestResourceWithStreamingResponse",
    "AsyncAutosuggestResourceWithStreamingResponse",
    "DirectionsResource",
    "AsyncDirectionsResource",
    "DirectionsResourceWithRawResponse",
    "AsyncDirectionsResourceWithRawResponse",
    "DirectionsResourceWithStreamingResponse",
    "AsyncDirectionsResourceWithStreamingResponse",
    "BatchResource",
    "AsyncBatchResource",
    "BatchResourceWithRawResponse",
    "AsyncBatchResourceWithRawResponse",
    "BatchResourceWithStreamingResponse",
    "AsyncBatchResourceWithStreamingResponse",
    "MultigeocodeResource",
    "AsyncMultigeocodeResource",
    "MultigeocodeResourceWithRawResponse",
    "AsyncMultigeocodeResourceWithRawResponse",
    "MultigeocodeResourceWithStreamingResponse",
    "AsyncMultigeocodeResourceWithStreamingResponse",
    "RevgeocodeResource",
    "AsyncRevgeocodeResource",
    "RevgeocodeResourceWithRawResponse",
    "AsyncRevgeocodeResourceWithRawResponse",
    "RevgeocodeResourceWithStreamingResponse",
    "AsyncRevgeocodeResourceWithStreamingResponse",
    "RouteReportResource",
    "AsyncRouteReportResource",
    "RouteReportResourceWithRawResponse",
    "AsyncRouteReportResourceWithRawResponse",
    "RouteReportResourceWithStreamingResponse",
    "AsyncRouteReportResourceWithStreamingResponse",
    "SnapToRoadsResource",
    "AsyncSnapToRoadsResource",
    "SnapToRoadsResourceWithRawResponse",
    "AsyncSnapToRoadsResourceWithRawResponse",
    "SnapToRoadsResourceWithStreamingResponse",
    "AsyncSnapToRoadsResourceWithStreamingResponse",
    "PostalcodeResource",
    "AsyncPostalcodeResource",
    "PostalcodeResourceWithRawResponse",
    "AsyncPostalcodeResourceWithRawResponse",
    "PostalcodeResourceWithStreamingResponse",
    "AsyncPostalcodeResourceWithStreamingResponse",
    "AreasResource",
    "AsyncAreasResource",
    "AreasResourceWithRawResponse",
    "AsyncAreasResourceWithRawResponse",
    "AreasResourceWithStreamingResponse",
    "AsyncAreasResourceWithStreamingResponse",
    "LookupResource",
    "AsyncLookupResource",
    "LookupResourceWithRawResponse",
    "AsyncLookupResourceWithRawResponse",
    "LookupResourceWithStreamingResponse",
    "AsyncLookupResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PostalcodeRetrieveCoordinatesParams", "At"]


class PostalcodeRetrieveCoordinatesParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    at: At
    """Location coordinates that you want to get the postal code of.

    If not providing `postalcode` in the request, `at` becomes mandatory. Please
    note that only 1 point can be requested. [See this example](#note).
    """

    country: str
    """Country containing the postal code or the location.

    It is mandatory if `postalcode` is provided in the request.
    [See this example](#note).

    Please check the [API Query Limits](#api-query-limits) section below for a list
    of the countries covered by the Geocode Postcode API. Users can provide either
    the name or the alpha-2/3 code as per the
    [ISO 3166-1 standard](https://en.wikipedia.org/wiki/ISO_3166-1) of a country
    covered by the API as input for this parameter.
    """

    format: Literal["`geojson`"]
    """
    Specify the format in which the boundary details of the post code will be
    returned. When specified, the boundary details will be returned in the `geojson`
    format. When not specified, the boundary details are returned in general format.
    """

    postalcode: str
    """Provide the postal code for which the information is needed.

    At least one of (`postalcode` + `country`) or `at` needs to be provided. Please
    note that only 1 postal code can be requested. [See this example](#note).
    """


class At(TypedDict, total=False):
    lat: float
    """Latitude of the location."""

    lng: float
    """Longitude of the location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["LocationListParams"]


class LocationListParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    correction: str
    """
    Describe the geometry characteristics through a `,` separated list of
    properties.

    Setting `mapmatch` to 1 returns the geometry of the tracked points, snapped to
    the nearest road.

    Setting `interpolate` to 1 smoothens the snapped geometry by adding more points,
    as needed. Please note, `mapmatch` should be set to 1 for `interpolate` to be
    effective.

    `mode` is used to set the transport mode for which the snapped route will be
    determined. Allowed values for `mode` are `car` and `truck`.
    """

    end_time: int
    """Time until which the tracked locations of the `asset` need to be retrieved."""

    geometry_type: Literal["`polyline`", "`polyline6`", "`geojson`"]
    """
    Set the geometry format to encode the path linking the tracked locations of the
    `asset`.

    Please note that `geometry_type` is effective only when `mapmatch` property of
    `correction` parameter is set to 1.
    """

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results.
    """

    start_time: int
    """Time after which the tracked locations of the `asset` need to be retrieved."""
